@@ -21,11 +21,14 @@
 #ifndef BASE_DB_H
 #define BASE_DB_H
 
-#include "gtkmm.h"
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
+#include <gtkmm.h>
 
 #include <glom/libglom/document/document_glom.h>
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/appstate.h>
+#include <glom/libglom/data_structure/foundset.h>
 #include <glom/libglom/data_structure/privileges.h>
 #include <glom/libglom/data_structure/system_prefs.h>
 #include <glom/libglom/utils.h>
@@ -40,28 +43,9 @@ class LayoutItem_GroupBy;
 class LayoutItem_Summary;
 class LayoutItem_VerticalGroup;
 
-class FoundSet
-{
-public:
-  FoundSet();
-  FoundSet(const FoundSet& src);
-  FoundSet& operator=(const FoundSet& src);
-
-  bool operator==(const FoundSet& src) const;
-
-  Glib::ustring m_table_name;
-  Glib::ustring m_extra_join; // Only used for doubly-related related records (portals), in which case the WHERE clause is also slightly different.
-  Glib::ustring m_where_clause;
-  Glib::ustring m_extra_group_by;  // Only used for doubly-related related records (portals), in which case the WHERE clause is also slightly different.
-
-  ///field, ascending
-  typedef std::pair< sharedptr<const LayoutItem_Field>, bool> type_pair_sort_field;
-  typedef std::list<type_pair_sort_field> type_sort_clause;
-  type_sort_clause m_sort_clause;
-};
 
 /** A base class that is a Bakery View with some database functionality.
-*/
+ */
 class Base_DB :
   public View_Composite_Glom
 {
@@ -72,16 +56,17 @@ public:
   /// Specify the structure of what will be shown, and fill it.
   virtual bool init_db_details();
 
-  /// Specify what actual data will be shown:
-  virtual bool refresh_data_from_database();
-
   /** Returns whether we are in developer mode.
    * Some functionality will be deactivated when not in developer mode.
    */
   virtual AppState::userlevels get_userlevel() const;
   virtual void set_userlevel(AppState::userlevels value);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   static sharedptr<SharedConnection> connect_to_server(Gtk::Window* parent_window = 0);
+#else
+  static sharedptr<SharedConnection> connect_to_server(Gtk::Window* parent_window, std::auto_ptr<ExceptionConnection>& error);
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
@@ -92,8 +77,9 @@ public:
   static bool get_field_exists_in_database(const Glib::ustring& table_name, const Glib::ustring& field_name);
 
 
-  //This is const because const means not changing this instance, not whether we change the database.
   static Glib::RefPtr<Gnome::Gda::DataModel> query_execute(const Glib::ustring& strQuery, Gtk::Window* parent_window = 0);
+  static int count_rows_returned_by(const Glib::ustring& sql_query);
+
 
   bool add_standard_groups();
   bool add_standard_tables() const;
@@ -112,10 +98,12 @@ public:
 protected:
   sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
   sharedptr<LayoutItem_Field> offer_field_list(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   sharedptr<LayoutItem_Field> offer_field_formatting(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
-  sharedptr<LayoutItem_Text> offer_textobject(const sharedptr<LayoutItem_Text>& start_textobject, Gtk::Window* transient_for = 0);
-  sharedptr<LayoutItem_Image> offer_imageobject(const sharedptr<LayoutItem_Image>& start_imageobject, Gtk::Window* transient_for = 0);
+  sharedptr<LayoutItem_Text> offer_textobject(const sharedptr<LayoutItem_Text>& start_textobject, Gtk::Window* transient_for = 0, bool show_title = true);
+  sharedptr<LayoutItem_Image> offer_imageobject(const sharedptr<LayoutItem_Image>& start_imageobject, Gtk::Window* transient_for = 0, bool show_title = true);
   sharedptr<LayoutItem_Notebook> offer_notebook(const sharedptr<LayoutItem_Notebook>& start_notebook, Gtk::Window* transient_for = 0);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   ///@result Whether the user would like to find again.
   static bool show_warning_no_records_found(Gtk::Window& transient_for);
@@ -136,6 +124,7 @@ protected:
   Glib::ustring get_find_where_clause_quick(const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search) const;
 
 
+  //Methods to be overridden by derived classes:
   virtual void set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value&  value);
   virtual void set_entered_field_data(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
 
@@ -278,9 +267,13 @@ protected:
   bool set_field_value_in_database(const LayoutFieldInRecord& field_in_record, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
   bool set_field_value_in_database(const LayoutFieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
 
-  //Get a single field value from the database.
+  ///Get a single field value from the database.
   Gnome::Gda::Value get_field_value_in_database(const LayoutFieldInRecord& field_in_record, Gtk::Window* parent_window);
 
+  ///Get a single field value from the database.
+  Gnome::Gda::Value get_field_value_in_database(const sharedptr<Field>& field, const FoundSet& found_set, Gtk::Window* parent_window);
+
+
 
   SystemPrefs get_database_preferences() const;
   void set_database_preferences(const SystemPrefs& prefs);
@@ -303,12 +296,14 @@ protected:
   bool check_entered_value_for_uniqueness(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value, Gtk::Window* parent_window);
   bool check_entered_value_for_uniqueness(const Glib::ustring& table_name, const Gtk::TreeModel::iterator& /* row */,  const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value, Gtk::Window* parent_window);
 
+  /** Fill the UI with information (data or structure, depending on the widget).
+   * Overridden by derived widgets to provide implementation.
+   */
   virtual bool fill_from_database();
-  virtual void fill_end(); //Call this from the end of fill_from_database() overrides.
 
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
 
-  type_vecLayoutFields get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document_Glom::type_mapLayoutGroupSequence& mapGroupSequence) const;
+  type_vecLayoutFields get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document_Glom::type_list_layout_groups& mapGroupSequence) const;
   void get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const sharedptr<LayoutGroup>& group, type_vecLayoutFields& vecFields) const;
 
   /** Get the relationship into which the row button should navigate,
@@ -321,13 +316,22 @@ protected:
 
   bool get_primary_key_is_in_foundset(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value);
 
+
+  /** A utility function to set the where_clause, even for doubly-related records.
+   *
+   * @param found_set The FoundSet to change.
+   * @param portal The related records portal whose records should be selected by the SQL query.
+   * @param foreign_key_value The value of the from field in the parent table.
+   */
+  void set_found_set_where_clause_for_portal(FoundSet& found_set, const sharedptr<LayoutItem_Portal>& portal, const Gnome::Gda::Value& foreign_key_value);
+
+
   static bool get_field_primary_key_index_for_fields(const type_vecFields& fields, guint& field_column);
   static bool get_field_primary_key_index_for_fields(const type_vecLayoutFields& fields, guint& field_column);
 
-
-
   static type_vecStrings util_vecStrings_from_Fields(const type_vecFields& fields);
 
+
   static void handle_error(const Glib::Exception& ex);
   static void handle_error(const std::exception& ex); //TODO_port: This is probably useless now.
   static bool handle_error();
