@@ -78,6 +78,7 @@ public:
 
   virtual ~sharedptr();
 
+  inline bool operator==(const sharedptr<T_obj>& src) const;
 
   ///Forget the instance.
   virtual void clear();
@@ -107,7 +108,17 @@ public:
    */
   inline operator bool() const;
 
-    /** Dynamic cast to derived class.
+  /** Test whether the sharedptr<> points to any underlying instance.
+   *
+   * Mimics usage of ordinary pointers:
+   * @code
+   *   if (!ptr)
+   *     do_something();
+   * @endcode
+   */
+  inline bool operator!() const;
+
+   /** Dynamic cast to derived class.
    *
    * The sharedptr can't be cast with the usual notation so instead you can use
    * @code
@@ -251,6 +262,13 @@ sharedptr<T_obj>::~sharedptr()
    unref();
 }
 
+template <class T_obj>
+inline bool sharedptr<T_obj>::operator==(const sharedptr<T_obj>& src) const
+{
+  return m_pobj == src.m_pobj;
+}
+
+
 template< typename T_obj>
 void sharedptr<T_obj>::clear()
 {
@@ -300,6 +318,12 @@ sharedptr<T_obj>::operator bool() const
   return (m_pobj != 0);
 }
 
+template <class T_obj>
+inline
+bool sharedptr<T_obj>::operator!() const
+{
+  return (m_pobj == 0);
+}
 
 template <class T_obj>
 inline
