@@ -24,7 +24,7 @@
 #include "../../frame_glom.h"
 #include "../../python_embed/glom_python.h"
 #include <glom/libglom/data_structure/glomconversions.h>
-#include <gtksourceviewmm/sourcelanguagesmanager.h>
+#include <gtksourceviewmm/sourcelanguagemanager.h>
 
 //#include <libgnome/gnome-i18n.h>
 #include <glibmm/i18n.h>
@@ -46,15 +46,14 @@ Dialog_FieldCalculation::Dialog_FieldCalculation(BaseObjectType* cobject, const
   {
     m_text_view->set_highlight_current_line(true);
 
-    //TODO: Shouldn't this be a singleton?
-    Glib::RefPtr<gtksourceview::SourceLanguagesManager> languages_manager = gtksourceview::SourceLanguagesManager::create();
+    Glib::RefPtr<gtksourceview::SourceLanguageManager> languages_manager = gtksourceview::SourceLanguageManager::get_default();
 
-    Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language_from_mime_type("application/x-python");
+    Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language("python"); //This is the GtkSourceView language ID.
     if(language)
     {
        //Createa a new buffer and set it, instead of getting the default buffer, in case libglade has tried to set it, using the wrong buffer type:
        Glib::RefPtr<gtksourceview::SourceBuffer> buffer = gtksourceview::SourceBuffer::create(language);
-       buffer->set_highlight();
+       buffer->set_highlight_syntax();
        m_text_view->set_buffer(buffer);
     }
   }
@@ -92,9 +91,22 @@ sharedptr<Field> Dialog_FieldCalculation::get_field() const
   return field;
 }
 
+bool Dialog_FieldCalculation::check_for_return_statement(const Glib::ustring& calculation)
+{
+  if(calculation.find("return") == Glib::ustring::npos)
+  {
+     Frame_Glom::show_ok_dialog(_("Calculation Error"), _("The calculation does not have a return statement."), *this);
+     return false;
+  }
+  
+  return true;
+}
+
 void Dialog_FieldCalculation::on_button_test()
 {
   const Glib::ustring calculation = m_text_view->get_buffer()->get_text();
+  if(!check_for_return_statement(calculation))
+    return;
 
   type_map_fields field_values;
 
@@ -113,7 +125,7 @@ void Dialog_FieldCalculation::on_button_test()
   //We need the connection when we run the script, so that the script may use it.
   sharedptr<SharedConnection> sharedconnection = connect_to_server(this /* parent window */);
 
-  Gnome::Gda::Value value = glom_evaluate_python_function_implementation(Field::TYPE_TEXT, calculation, field_values, //TODO: Maybe use the field's type here.
+  const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(Field::TYPE_TEXT, calculation, field_values, //TODO: Maybe use the field's type here.
     document, m_table_name, sharedconnection->get_gda_connection());
 
   Frame_Glom::show_ok_dialog(_("Calculation result"), _("The result of the calculation is:\n") + value.to_string(), *this);
