@@ -21,12 +21,14 @@
 #ifndef NOTEBOOK_GLOM_H
 #define NOTEBOOK_GLOM_H
 
-#include "box_db.h"
+#include "box_withbuttons.h"
 #include <glom/libglom/document/document_glom.h>
 
-/**Notebook with document methods.
-  */
+namespace Glom
+{
 
+/** Notebook with document methods.
+  */
 class Notebook_Glom :
   public Gtk::Notebook,
   public Base_DB
@@ -40,23 +42,29 @@ public:
   //Signals:
   //Page number
   //typedef sigc::signal<void, guint> type_signal_leave_page;
- // type_signal_leave_page signal_leave_page();
+  // type_signal_leave_page signal_leave_page();
 
- virtual void do_menu_developer_layout();
- virtual void do_menu_file_print();
+  //Overridden by derived classes:
+  virtual void do_menu_developer_layout();
+  virtual void do_menu_file_print();
 
 protected:
 
+  void on_show();
+
   Gtk::Window* get_app_window();
 
   //Signal handlers:
   virtual void on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber);  //The _handler suffix is to avoid overriding the base class's method.
-  virtual void on_leave_page(guint uiPageNumber);
+  void on_leave_page(guint uiPageNumber);
 
   //type_signal_leave_page m_signal_leave_page; //Signals when the user leaves a page.
 
   guint m_uiPreviousPage;
   bool m_destructor_in_progress; //A hack to prevent calling wrap() on dead C instances.
+  sigc::connection m_connection_switch_page; //This allows us to delay connecting, and to block the handler temporarily.
 };
 
+} //namespace Glom
+
 #endif
