@@ -28,6 +28,7 @@
 #include <glom/libglom/data_structure/layout/layoutitem_field.h>
 #include <glom/libglom/data_structure/layout/layoutitem_text.h>
 #include <glom/libglom/data_structure/layout/layoutitem_image.h>
+#include <glom/libglom/glade_utils.h>
 #include "../mode_data/dialog_choose_field.h"
 #include "../layout_item_dialogs/dialog_field_layout.h"
 #include "../layout_item_dialogs/dialog_group_by.h"
@@ -272,9 +273,9 @@ void Dialog_Layout_Report::fill_group_children(const sharedptr<LayoutGroup>& gro
 
 void Dialog_Layout_Report::add_group_children(const Glib::RefPtr<type_model>& model_parts, const Gtk::TreeModel::iterator& parent, const sharedptr<const LayoutGroup>& group)
 {
-  for(LayoutGroup::type_map_items::const_iterator iter = group->m_map_items.begin(); iter != group->m_map_items.end(); ++iter)
+  for(LayoutGroup::type_list_items::const_iterator iter = group->m_list_items.begin(); iter != group->m_list_items.end(); ++iter)
   {
-    sharedptr<const LayoutItem> item = iter->second;
+    sharedptr<const LayoutItem> item = *iter;
     sharedptr<const LayoutGroup> group = sharedptr<const LayoutGroup>::cast_dynamic(item);
     if(group)
     {
@@ -660,7 +661,7 @@ void Dialog_Layout_Report::on_button_add()
     Gtk::TreeModel::iterator iter;
     if(parent)
     {
-      m_treeview_parts_main->expand_row( Gtk::TreePath(parent), true);
+      m_treeview_parts_main->expand_row( Gtk::TreeModel::Path(parent), true);
       iter = model->append(parent->children());
     }
     else
@@ -670,7 +671,7 @@ void Dialog_Layout_Report::on_button_add()
   }
 
   if(parent)
-    treeview->expand_row( Gtk::TreePath(parent), true);
+    treeview->expand_row( Gtk::TreeModel::Path(parent), true);
 
   enable_buttons();
 }
@@ -682,7 +683,7 @@ sharedptr<Relationship> Dialog_Layout_Report::offer_relationship_list()
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_relationship");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_relationship");
 
     Dialog_ChooseRelationship* dialog = 0;
     refXml->get_widget_derived("dialog_choose_relationship", dialog);
@@ -788,7 +789,7 @@ void Dialog_Layout_Report::on_button_formatting()
         if(field_chosen)
         {
           *field = *field_chosen;
-          model->row_changed(Gtk::TreePath(iter), iter);
+          model->row_changed(Gtk::TreeModel::Path(iter), iter);
         }
       }
     }
@@ -819,7 +820,7 @@ void Dialog_Layout_Report::on_button_edit()
       {
         try
         {
-          Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_field_summary");
+          Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_field_summary");
 
           Dialog_FieldSummary* dialog = 0;
           refXml->get_widget_derived("dialog_field_summary", dialog);
@@ -841,7 +842,7 @@ void Dialog_Layout_Report::on_button_edit()
               {
                 *fieldsummary = *chosenitem; //TODO_Performance.
 
-                model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                 m_modified = true;
               }
             }
@@ -864,7 +865,7 @@ void Dialog_Layout_Report::on_button_edit()
           if(chosenitem)
           {
             *field = *chosenitem; //TODO_Performance.
-            model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+            model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
             m_modified = true;
           }
         }
@@ -877,7 +878,7 @@ void Dialog_Layout_Report::on_button_edit()
             if(chosen)
             {
               *layout_item_text = *chosen;
-              model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+              model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
               m_modified = true;
             }
           }
@@ -890,7 +891,7 @@ void Dialog_Layout_Report::on_button_edit()
               if(chosen)
               {
                 *layout_item_image = *chosen;
-                model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                 m_modified = true;
               }
             }
@@ -901,7 +902,7 @@ void Dialog_Layout_Report::on_button_edit()
               {
                 try
                 {
-                  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_group_by");
+                  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_group_by");
   
                   Dialog_GroupBy* dialog = 0;
                   refXml->get_widget_derived("dialog_group_by", dialog);
@@ -922,7 +923,7 @@ void Dialog_Layout_Report::on_button_edit()
                       if(chosenitem)
                       {
                         *group_by = *chosenitem;
-                        model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                        model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                         m_modified = true;
                       }
                     }
@@ -947,48 +948,6 @@ void Dialog_Layout_Report::on_button_edit()
 void Dialog_Layout_Report::save_to_document()
 {
   Dialog_Layout::save_to_document();
-
-/*
-  if(m_modified)
-  {
-    //Set the table name and title:
-    Document_Glom* document = get_document();
-    //if(document)
-    //  document->set_table_title( m_table_name, m_entry_table_title->get_text() );
-
-    //Get the data from the TreeView and store it in the document:
-
-    //Fill the sequences:
-    //(This model is not sorted - we need to set the sequence numbers based on the order).
-    //m_model_parts_main->fill_sequences();
-
-    //Get the groups and their fields:
-    Document_Glom::type_mapLayoutGroupSequence mapGroups;
-    guint group_sequence = 1; //0 means no sequence
-
-    //Add the layout items:
-    //guint field_sequence = 1; //0 means no sequence
-    for(Gtk::TreeModel::iterator iterFields = m_model_parts_main->children().begin(); iterFields != m_model_parts_main->children().end(); ++iterFields)
-    {
-      //Gtk::TreeModel::Row row = *iterFields;
-      //if(row[m_columns_parts->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP) //There may be top-level groups, but no top-level fields, because the fields must be in a group (so that they are in columns)
-      {
-        LayoutGroup group;
-        group->m_sequence = group_sequence;
-        fill_group(iterFields, group);
-
-        mapGroups[group_sequence] = group;
-        ++group_sequence;
-      }
-    }
-
-    if(document)
-    {
-      document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
-      m_modified = false;
-    }
-  }
-*/
 }
 
 void Dialog_Layout_Report::on_treeview_available_parts_selection_changed()
