@@ -19,8 +19,12 @@
  */
  
 #include "flowtable.h"
-#include "gtk/gtkwidget.h"
-#include "gdk/gdktypes.h"
+#include "layoutwidgetbase.h"
+#include <gtk/gtkwidget.h>
+#include <gdk/gdktypes.h>
+#include <iostream>
+#include <gdkmm/window.h>
+#include <gtk/gtk.h>
 
 // So we don't need to check for the condition above all the time
 
@@ -33,7 +37,9 @@ namespace
 #define GLOM_FLOWTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), GLOM_TYPE_FLOWTABLE, GlomFlowtable))
 #endif
 
+#ifndef GLIBMM_VFUNCS_ENABLED
   GtkContainerClass* parent_class = NULL;
+#endif
 
 #if 0
   struct GlomFlowtableClass
@@ -246,12 +252,47 @@ namespace Glom
 namespace Glom
 {
 
-FlowTable::FlowTableItem::FlowTableItem()
-: m_expand_first_full(false),
+static void container_forall_callback(GtkWidget* widget_gobj, void* data)
+{
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
+    FlowTable::ForallSlot& slot = *static_cast<FlowTable::ForallSlot*>(data);
+    Gtk::Widget *const widget = Glib::wrap(widget_gobj);
+
+    g_return_if_fail(widget != 0);
+
+    slot(*widget);
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
+  }
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
+}
+
+  
+FlowTable::FlowTableItem::FlowTableItem(Gtk::Widget* first, FlowTable* flowtable)
+: m_first(first),
+  m_second(0),
+  m_expand_first_full(false),
   m_expand_second(false)
 {
+
 }
 
+FlowTable::FlowTableItem::FlowTableItem(Gtk::Widget* first, Gtk::Widget* second, FlowTable* flowtable)
+: m_first (first),
+  m_second(second),
+  m_expand_first_full(false),
+  m_expand_second(false)
+{
+
+}
+
+
 FlowTable::FlowTable()
 :
 #if !defined(GLIBMM_VFUNCS_ENABLED) || !defined(GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED)
@@ -301,6 +342,33 @@ FlowTable::FlowTable()
 
 FlowTable::~FlowTable()
 {
+  //Delete managed children.
+  //(For some reason this is not always happening via Gtk::Container:
+  /* Actualy, don't do this because we would then be double-deleting what the 
+   * Container base class already deleted. We'll have to really find out if/why some 
+   * things are not being deleted. murrayc.
+  bool one_deleted = true;
+  while(!m_children.empty() && one_deleted)
+  {
+    one_deleted = false;
+
+    type_vecChildren::iterator iter = m_children.begin();
+    FlowTableItem& item = *iter;
+
+    //Delete the widgets:
+    if(item.m_first || item.m_second)
+    {
+      if(item.m_first && item.m_first->is_managed_())
+        delete item.m_first;
+
+      if(item.m_second && item.m_second->is_managed_())
+        delete item.m_second;
+
+      m_children.erase(iter);
+      one_deleted = true; //Make sure that we loop again.
+    }
+  }
+  */
 }
 
 void FlowTable::set_design_mode(bool value)
@@ -312,25 +380,84 @@ void FlowTable::set_design_mode(bool value)
 
 void FlowTable::add(Gtk::Widget& first, Gtk::Widget& second, bool expand_second)
 {
-  FlowTableItem item;
-  item.m_first = &first;
-  item.m_second = &second;
+  FlowTableItem item (&first, &second, this);
+  
   item.m_expand_second = expand_second; //Expand to fill the width for all of the second item.
   m_children.push_back(item);
-
-  gtk_widget_set_parent(first.gobj(), GTK_WIDGET(gobj()));
-  gtk_widget_set_parent(second.gobj(), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_second->gobj()), GTK_WIDGET(gobj()));
 }
 
 void FlowTable::add(Gtk::Widget& first, bool expand)
 {
-  FlowTableItem item;
-  item.m_first = &first;
+  FlowTableItem item(&first, this);
   item.m_expand_first_full = expand; //Expand to fill the width for first and second.
-  item.m_second = 0;
   m_children.push_back(item);
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+}
 
-  gtk_widget_set_parent(first.gobj(), GTK_WIDGET(gobj()));
+void FlowTable::insert_before(Gtk::Widget& first, Gtk::Widget& before, bool expand)
+{
+  FlowTableItem item(&first, this);
+  item.m_expand_first_full = expand;
+  insert_before (item, before);
+}
+
+void FlowTable::insert_before(Gtk::Widget& first, Gtk::Widget& second, Gtk::Widget& before, bool expand_second)
+{
+  FlowTableItem item(&first, &second, this);
+  item.m_expand_second = expand_second;
+  insert_before (item, before);
+}
+
+void FlowTable::insert_before(FlowTableItem& item, Gtk::Widget& before)
+{
+  bool found = false;
+  std::vector<FlowTableItem>::iterator pos;
+  for (pos = m_children.begin(); pos != m_children.end(); pos++)
+  {
+    FlowTableItem* item = &(*pos);
+    if (item->m_first)
+    {
+      if (item->m_first->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+      Gtk::Alignment* alignment = dynamic_cast<Gtk::Alignment*>(item->m_first);
+      if (alignment && alignment->get_child()->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+    }
+    if (item->m_second)
+    {
+      if (item->m_second->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+      Gtk::Alignment* alignment = dynamic_cast<Gtk::Alignment*>(item->m_second);
+      if (alignment && alignment->get_child()->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+    }    
+  }
+ 
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+  if (item.m_second)
+  {
+    gtk_widget_set_parent(GTK_WIDGET (item.m_second->gobj()), GTK_WIDGET(gobj()));
+  }
+  if (pos == m_children.end())
+  {
+    m_children.push_back(item);
+  } else {
+    m_children.insert(pos, item);
+  }
 }
 
 void FlowTable::set_columns_count(guint value)
@@ -419,7 +546,7 @@ int FlowTable::get_column_height(guint start_widget, guint widget_count, int& to
   guint i = 0;
   for(i = start_widget; i < (start_widget+widget_count);  ++i)
   {
-    FlowTableItem item = m_children[i];
+    const FlowTableItem& item = m_children[i];
     int item_height = get_item_requested_height(item);
 
     int item_width_first = 0;
@@ -531,6 +658,11 @@ int FlowTable::get_minimum_column_height(guint start_widget, guint columns_count
 
 void FlowTable::on_size_request(Gtk::Requisition* requisition)
 {
+  // Set a minimum size so people are able to drag items into the
+  // table
+  const int MIN_HEIGHT = (m_design_mode ? 50 : 0);
+  const int MIN_WIDTH = (m_design_mode ? 100 : 0);
+  
   //Initialize the output parameter:
   *requisition = Gtk::Requisition();
 
@@ -539,8 +671,8 @@ void FlowTable::on_size_request(Gtk::Requisition* requisition)
   int total_width = 0;
   const int column_height = get_minimum_column_height(0, m_columns_count, total_width); //This calls itself recursively.
 
-  requisition->height = column_height;
-  requisition->width = total_width;
+  requisition->height = (column_height > MIN_HEIGHT ? column_height : MIN_HEIGHT);
+  requisition->width = (total_width > MIN_WIDTH ? total_width : MIN_WIDTH);
 }
 
 //Give it whatever height/width it wants, at this location:
@@ -590,7 +722,7 @@ void FlowTable::get_item_max_width_requested(guint start, guint height, guint& f
   guint i = start;
   while( (height_so_far < height) && (i < m_children.size()))
   {
-    FlowTableItem item = m_children[i];
+    const FlowTableItem& item = m_children[i];
     Gtk::Widget* first = item.m_first;
     Gtk::Widget* second = item.m_second;
 
@@ -699,7 +831,7 @@ void FlowTable::on_size_allocate(Gtk::Allocation& allocation)
   {
     FlowTableItem& item = m_children[i];
 
-    int item_height = get_item_requested_height(item);
+    const int item_height = get_item_requested_height(item);
 
     //Start a new column if necessary:
     int bottom = allocation.get_y() + allocation.get_height();   
@@ -816,12 +948,10 @@ GtkType FlowTable::child_type_vfunc() const
 
 void FlowTable::on_add(Gtk::Widget* child)
 {
-  FlowTableItem item;
-  item.m_first = child;
-  item.m_second = 0;
+  FlowTableItem item(child, 0);
   m_children.push_back(item);
   
-  gtk_widget_set_parent(child->gobj(), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
 
   //This is protected, but should be public: child.set_parent(*this);
 
@@ -842,7 +972,7 @@ void FlowTable::on_remove(Gtk::Widget* child)
       if(item.m_first == child)
       {
             //g_warning("FlowTable::on_remove unparenting first");
-        gtk_widget_unparent(item.m_first->gobj()); //This is protected, but should be public: child.unparent();
+        gtk_widget_unparent(GTK_WIDGET (item.m_first->gobj())); //This is protected, but should be public: child.unparent();
         item.m_first = 0;
 
        if(visible)
@@ -852,7 +982,7 @@ void FlowTable::on_remove(Gtk::Widget* child)
       if(item.m_second == child)
       {
         //g_warning("FlowTable::on_remove unparenting second");
-        gtk_widget_unparent(item.m_second->gobj()); //This is protected, but should be public: child.unparent();
+        gtk_widget_unparent(GTK_WIDGET (item.m_second->gobj())); //This is protected, but should be public: child.unparent();
         item.m_second = 0;
 
         if(visible)
@@ -881,6 +1011,12 @@ void FlowTable::forall_vfunc(gboolean /* include_internals */, GtkCallback callb
   }
 }
 
+void FlowTable::forall(const ForallSlot& slot)
+{
+  ForallSlot slot_copy (slot);
+  gtk_container_forall(gobj(), &container_forall_callback, &slot_copy);
+}
+
 /** Sets the padding to put between the child widgets.
  */
 void FlowTable::set_padding(guint padding)
@@ -925,7 +1061,7 @@ void FlowTable::remove_all()
       if(widget->is_managed_())
         widget->reference();
 
-      gtk_widget_unparent(widget->gobj());
+      gtk_widget_unparent(GTK_WIDGET (iter->m_first->gobj()));
     }
 
     if(iter->m_second)
@@ -935,7 +1071,7 @@ void FlowTable::remove_all()
       if(widget->is_managed_())
         widget->reference();
 
-      gtk_widget_unparent(widget->gobj());
+      gtk_widget_unparent(GTK_WIDGET (iter->m_second->gobj()));
     }
 
   }
@@ -982,16 +1118,6 @@ bool FlowTable::on_expose_event(GdkEventExpose* event)
       m_refGC = Gdk::GC::create(m_refGdkWindow);
       m_refGC->set_line_attributes(1 /* width */, Gdk::LINE_ON_OFF_DASH, Gdk::CAP_NOT_LAST, Gdk::JOIN_MITER);
 
-      //Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap();
-      //Gdk::Color color_blue("blue");
-      //Gdk::Color color_red("red");
-      //colormap->alloc_color(color_blue);
-      //colormap->alloc_color(color_red);
-
-      //m_refGdkWindow->set_background( color_red );
-      //m_refGdkWindow->clear();
-      //m_refGC->set_foreground( color_blue );
-
       for(type_vecLines::iterator iter = m_lines_horizontal.begin(); iter != m_lines_horizontal.end(); ++iter)
       {
         //TODO: Add draw_line(point, point) to gdkmm:
@@ -1004,7 +1130,6 @@ bool FlowTable::on_expose_event(GdkEventExpose* event)
       }
     }
   }
-
 #ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   return Gtk::Container::on_expose_event(event);
 #else
